/*******************************************************************************
 * (c) Copyright 2012 Microsemi Corporation.  All rights reserved.
 *
 *  Microsemi Smartfusion2 USB device logical driver layer Implementation.	
 *  
 *
 *  This file implements CDC class specific initialization and request handling.
 *
 * SVN $Revision: 5027 $
 * SVN $Date: 2013-02-01 19:06:39 +0530 (Fri, 01 Feb 2013) $
 */

#include "mss_usb_device.h"
#include "mss_usb_device_cdc.h"

#ifdef __cplusplus
extern "C" {
#endif 

/*******************************************************************************
 * Local Function
 */

uint8_t* MSS_USB_CDC_get_descriptor_cb(uint8_t recepient, uint8_t type, uint32_t* length);
uint8_t MSS_USB_CDC_init_cb(uint8_t cfgidx);
uint8_t MSS_USB_CDC_deinit_cb(uint8_t cfgidx);
uint8_t MSS_USB_CDC_process_request_cb(uint8_t** buf_pp,
                                    uint8_t request,
                                    uint32_t* length,
                                    uint8_t* data_buf_p,
                                    uint32_t data_len);
uint8_t MSS_USB_CDC_datain_cb(mss_usb_ep_num_t num, uint8_t status);
uint8_t MSS_USB_CDC_dataout_cb(mss_usb_ep_num_t num, uint8_t status, uint32_t rx_count);
uint8_t MSS_USB_CDC_cep_datain_cb(uint8_t status);
uint8_t MSS_USB_CDC_cep_dataout_cb(uint8_t status);

usb_cdc_state_t g_usb_cdc_state = USB_CDC_NOT_CONFIGURED;

mss_usb_class_cb_t usb_cdc_class_cb = {
    MSS_USB_CDC_init_cb,
    MSS_USB_CDC_deinit_cb,
    MSS_USB_CDC_get_descriptor_cb,
    MSS_USB_CDC_process_request_cb,
    MSS_USB_CDC_datain_cb,
    MSS_USB_CDC_dataout_cb,
    MSS_USB_CDC_cep_datain_cb,
    MSS_USB_CDC_cep_dataout_cb
};

mss_usb_cdc_app_cb_t* mss_usb_cdc_vcp_ops;

uint8_t cdc_configuration_descriptor[FULL_CONFIG_DESCR_LENGTH] = {
    
    /*----------------------- Configuration Descriptor -----------------------*/
    CONFIGURATION_DESCR_LENGTH,                         /* bLength */
    USB_CONFIGURATION_DESCRIPTOR_TYPE,                  /* bDescriptorType */
    FULL_CONFIG_DESCR_LENGTH,                           /* wTotalLength LSB */
    0x00u,                                              /* wTotalLength MSB */
    0x02u,                                              /* bNumInterfaces */	
                                                        /*interface number (which starts with 0) + 1
                                                         Comm Data Interface + Comm Management interface.*/
    0x01u,                                              /* bConfigurationValue */
    0x00u,                                              /* iConfiguration */
    0xC0u,                                              /* bmAttributes */
    0x32u,                                              /* bMaxPower */	

    /*------------------------- Interface Association Descriptor -------------------------*/

	INTERFACE_ASSOCIATION_DESCR_LENGTH,  	            /* bLength*/
	USB_INTERFACE_ASSOCIATION_DESCRIPTOR_TYPE,          /* bDescriptorType = 11u*/
	0x00,									            /* bFirstInterface*/
	0x02,									            /* bInterfaceCount*/
	0x02,									            /* bFunctionClass (Communication Class)*/
	0x02,									            /* bFunctionSubClass (Abstract Control Model)*/
	0x01,									            /* bFunctionProcotol (V250 (formerly V.25ter), Common AT commands)*/
	0x00,									            /* iInterface*/

    /*------------------------- Interface 0 - Communications management Descriptor -------------------------*/
    INTERFACE_DESCR_LENGTH,                             /* bLength */
    USB_INTERFACE_DESCRIPTOR_TYPE,                      /* bDescriptorType */
	0x00,								                /* bInterfaceNumber: Number of this Interface */
	0x00,								                /* bAlternateSetting: Alternate setting */
	0x01,								                /* bNumEndpoints: One endpoints used */
	0x02,								                /* bInterfaceClass: Communication Interface Class */
	0x02,								                /* bInterfaceSubClass: Abstract Control Model */
	0x01,								                /* bInterfaceProtocol: Common AT commands */
	0x00,								                /* iInterface: */

	/*-------------------------Header Functional Descriptor-------------------------*/
	0x05,								                /* bLength: Endpoint Descriptor size */
	0x24,								                /* bDescriptorType: CS_INTERFACE */
	0x00,								                /* bDescriptorSubtype: Header Func Desc */
	0x10,								                /* bcdCDC: spec release number */
	0x01,
	
	/*-------------------------Call Management Functional Descriptor-------------------------*/
	0x05,								                /* bFunctionLength */
	0x24,								                /* bDescriptorType: CS_INTERFACE */
	0x01,								                /* bDescriptorSubtype: Call Management Func Desc */
	0x00,								                /* bmCapabilities: D0+D1 */
	0x01,								                /* bDataInterface: 1 */
	
	/*-------------------------ACM Functional Descriptor-------------------------*/
	0x04,								                /* bFunctionLength */
	0x24,								                /* bDescriptorType: CS_INTERFACE */
	0x02,								                /* bDescriptorSubtype: Abstract Control Management desc */
	0x02,								                /* bmCapabilities */
	
	/*-------------------------Union Functional Descriptor-------------------------*/
	0x05,								                /* bFunctionLength */
	0x24,								                /* bDescriptorType: CS_INTERFACE */
	0x06,								                /* bDescriptorSubtype: Union func desc */
	0x00,								                /* bMasterInterface: Communication class interface */
	0x01,								                /* bSlaveInterface0: Data Class Interface */

	/*-------------------------Communications management Endpoint Descriptor-------------------------*/
	ENDPOINT_DESCR_LENGTH,				                /* bLength */
	USB_ENDPOINT_DESCRIPTOR_TYPE,		                /* bDescriptorType */
	0x81u,								                /* bEndpointAddress */		/*TX EP 1 */
	0x03u,								                /* bmAttributes */	/*DSC_EP_INTERRUPT*/
	0x40u,								                /* wMaxPacketSize LSB */	/*Max Pkt Sz = 64*/
	0x00u,								                /* wMaxPacketSize MSB */
	0x10u,								                /* bInterval 0x10 -- corresponds to 4mSec.*/ 	

	/*------------------------- Interface 1 - Data class interface descriptor -------------------------*/
	0x09,								                /* bLength: Endpoint Descriptor size */
	USB_INTERFACE_DESCRIPTOR_TYPE,		                /* bDescriptorType: */
	0x01,								                /* bInterfaceNumber: Number of this Interface */
	0x00,								                /* bAlternateSetting: Alternate setting */
	0x02,								                /* bNumEndpoints: Two endpoints used */
	0x0A,								                /* bInterfaceClass: CDC */
	0x00,								                /* bInterfaceSubClass: */
	0x00,								                /* bInterfaceProtocol: */
	0x00,								                /* iInterface: */
	
	/*-------------------------Endpoint OUT Descriptor-------------------------*/
	ENDPOINT_DESCR_LENGTH,				                /* bLength: Endpoint Descriptor size */
	USB_ENDPOINT_DESCRIPTOR_TYPE,	   	                /* bDescriptorType: Endpoint */
	0x01, 					   			                /* bEndpointAddress */  /*RX_EP 1*/
	0x02,							   	                /* bmAttributes: Bulk */
	0x00,							   	                /* wMaxPacketSize: */
	0x02,
	0x00,							   	                /* bInterval: ignored for Bulk transfer */
	
	/*-------------------------Endpoint IN Descriptor-------------------------*/
	ENDPOINT_DESCR_LENGTH,				                /* bLength: Endpoint Descriptor size */
	USB_ENDPOINT_DESCRIPTOR_TYPE,	  	                /* bDescriptorType: Endpoint */
	0x82,						  		                /* bEndpointAddress */		/*TX_EP 2*/
	0x02,							  	                /* bmAttributes: Bulk */
	0x00,							  	                /* wMaxPacketSize: */
	0x02,
	0x00							  	                /* bInterval: ignored for Bulk transfer */
};

uint8_t*
MSS_USB_CDC_get_descriptor_cb
(
	uint8_t recepient,
	uint8_t type,
	uint32_t* length
)
{
	if((recepient == USB_STD_REQ_RECIPIENT_DEVICE) &&
        (type == USB_CONFIGURATION_DESCRIPTOR_TYPE))
    {
	   *length = sizeof(cdc_configuration_descriptor);
        return(cdc_configuration_descriptor);
	}
	if(recepient == USB_STD_REQ_RECIPIENT_ENDPOINT)		/*Need index(EP Num) here*/
	{

	}
	if(recepient == USB_STD_REQ_RECIPIENT_INTERFACE)	/*Need index(intrfs num) here*/
	{
	}
    return FAIL;
}

uint8_t
MSS_USB_CDC_init_cb
(
	uint8_t cfgidx
)
{
    uint8_t *buf_p;
    uint32_t len;

	MSS_USB_device_tx_ep_configure(VCP_INTR_COMM_IN_EP,
                                   VCP_COMM_TX_EP_FIFO_ADDR,
                                   VCP_COMM_TX_EP_FIFO_SIZE,
                                   VCP_COMM_TX_EP_MAX_PKT_SIZE,
                                   1,
                                   DMA_DISABLE,
                                   MSS_USB_DMA_CHANNEL1,
                                   MSS_USB_XFR_INTERRUPT,
                                   NO_ZLP_TO_XFR);

    MSS_USB_device_tx_ep_configure(VCP_BULK_DATA_IN_EP,
                                   VCP_DATA_TX_EP_FIFO_ADDR,
                                   VCP_DATA_TX_EP_FIFO_SIZE,
                                   VCP_DATA_TX_EP_MAX_PKT_SIZE,
                                   1,
                                   DMA_DISABLE,
                                   MSS_USB_DMA_CHANNEL2,
                                   MSS_USB_XFR_BULK,
                                   ADD_ZLP_TO_XFR);

    MSS_USB_device_rx_ep_configure(VCP_BULK_DATA_OUT_EP,
                                   VCP_DATA_RX_EP_FIFO_ADDR,
                                   VCP_DATA_RX_EP_FIFO_SIZE,
                                   VCP_DATA_RX_EP_MAX_PKT_SIZE,
                                   1,
                                   DMA_DISABLE,
                                   MSS_USB_DMA_CHANNEL3,
                                   MSS_USB_XFR_BULK,
                                   ADD_ZLP_TO_XFR);
    
    g_usb_cdc_state = USB_CDC_CONFIGURED;
	mss_usb_cdc_vcp_ops->usb_cdc_init();
	
	mss_usb_cdc_vcp_ops->usb_cdc_notification(&buf_p, &len);

	if(buf_p)
	{
		MSS_USB_device_tx_ep_write(VCP_INTR_COMM_IN_EP, buf_p, len);	
	}
	return 1;
}

uint8_t
MSS_USB_CDC_deinit_cb
(
	uint8_t cfgidx
)
{
	return 1;
}

uint8_t
MSS_USB_CDC_process_request_cb
(
	uint8_t** buf_pp, 
	uint8_t request,
	uint32_t* length,
	uint8_t* data_buf_p,
	uint32_t data_len	
)
{
	if(SUCCESS == mss_usb_cdc_vcp_ops->usb_cdc_process_request(buf_pp,
                                                       request,
                                                       length,
                                                       data_buf_p,
                                                       data_len))
    {
        return SUCCESS;
    }
    else
    {
        return FAIL;
    }
}  

uint8_t
MSS_USB_CDC_datain_cb
(
	mss_usb_ep_num_t num,
	uint8_t status
)
{
    uint8_t *buf_p;
    uint32_t len;
    if(num == VCP_INTR_COMM_IN_EP)
	{
    	mss_usb_cdc_vcp_ops->usb_cdc_notification(&buf_p, &len);
        if(buf_p)
	    {
	    	MSS_USB_device_tx_ep_write(VCP_INTR_COMM_IN_EP, buf_p, len);	
	    }
	}
	else if(num == VCP_BULK_DATA_IN_EP)
	{
    	mss_usb_cdc_vcp_ops->usb_cdc_tx_complete(status);
	}
    else
    {
        return FAIL;    
    }
    return SUCCESS;
}

uint8_t
MSS_USB_CDC_dataout_cb
(
	mss_usb_ep_num_t num,
	uint8_t status,
	uint32_t rx_count
)
{
	if(num == VCP_BULK_DATA_OUT_EP)
	{
    	mss_usb_cdc_vcp_ops->usb_cdc_rx(status, rx_count);
	}
    else
    {
        return FAIL;    
    }
    return SUCCESS;
}

uint8_t
MSS_USB_CDC_cep_datain_cb
(
	uint8_t status
)
{
    return SUCCESS;    
}
uint8_t
MSS_USB_CDC_cep_dataout_cb
(
    uint8_t status
)
{
    return SUCCESS;    
}

/*******************************************************************************
 * Exported functions from CDC class driver.
 ******************************************************************************/

/***************************************************************************//**
* See mss_usb_device_cdc.h for details of how to use this function.
*/
void
MSS_USB_CDC_init
(
    mss_usb_cdc_app_cb_t* vcp_app_cb
)
{
    mss_usb_cdc_vcp_ops = vcp_app_cb;
    MSS_USB_device_set_class_cb_handler(&usb_cdc_class_cb);
}

/***************************************************************************//**
* See mss_usb_device_cdc.h for details of how to use this function.
*/
void
MSS_USB_CDC_tx
(
	uint8_t* buf,
	uint32_t length
)
{
	MSS_USB_device_tx_ep_write(VCP_BULK_DATA_IN_EP,
                               buf,
                               length);
    
    g_usb_cdc_state = USB_CDC_TRANSMITTING;
}

/***************************************************************************//**
* See mss_usb_device_cdc.h for details of how to use this function.
*/
void
MSS_USB_CDC_rx_prepare
(
	uint8_t* buf,
	uint32_t length
)
{
	MSS_USB_device_rx_ep_read_prepare(VCP_BULK_DATA_OUT_EP,
                                      buf,
                                      length);
    
    g_usb_cdc_state = USB_CDC_RECEIVING;
}

/***************************************************************************//**
* See mss_usb_device_cdc.h for details of how to use this function.
*/
usb_cdc_state_t
MSS_USB_CDC_get_state
(
	void
)
{
	return g_usb_cdc_state;
}

#ifdef __cplusplus
}
#endif
