/*******************************************************************************
 * (c) Copyright 2012 Microsemi Corporation.  All rights reserved.
 *
 *  Microsemi Smartfusion2 USB device logical driver layer Implementation.	
 *  
 *
 *  Header file for mss_usb_device_msd.c
 *
 * SVN $Revision: 5027 $
 * SVN $Date: 2013-02-01 19:06:39 +0530 (Fri, 01 Feb 2013) $
 */

#ifndef __MSS_USB_DEVICE_MSD_H_
#define __MSS_USB_DEVICE_MSD_H_

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

/*******************************************************************************
 * User Descriptor lengths
 */
#define CONFIGURATION_DESCR_LENGTH                      9u
#define INTERFACE_DESCR_LENGTH                          9u
#define MSD_DESCR_LENGTH                                9u
#define ENDPOINT_DESCR_LENGTH                           7u


/* Full configuration descriptor length. APPLICATION/CONFIGURATION SPECIFIC!!!*/
#define FULL_CONFIG_DESCR_LENGTH                        ( CONFIGURATION_DESCR_LENGTH + \
                                                        INTERFACE_DESCR_LENGTH + \
                                                        ENDPOINT_DESCR_LENGTH + ENDPOINT_DESCR_LENGTH )

/***************************************************************************//**
  BOT SPecific definitions.
 */

#define USB_MSD_BOT_CSW_LENGTH                          0x0Du
#define USB_MSD_BOT_CBW_LENGTH		                    31u
#define USB_MSD_INQUIRY_RESPONSE_LENGTH                 36u


#define USB_MSD_BOT_REQ_GET_MAX_LUN                     0xFEu
#define USB_MSD_BOT_REQ_BMS_RESET			            0xFFu

#define USB_MSD_BOT_CBW_SIGNATURE                       0x43425355u
#define USB_MSD_BOT_CSW_SIGNATURE                       0x53425355u

/***************************************************************************//**
  Supported SCSI commands.
 */
#define USB_MSD_SCSI_INQUIRY                            0x12u
#define USB_MSD_SCSI_READ_FORMAT_CAPACITIES             0x23u
#define USB_MSD_SCSI_READ_CAPACITY_10                   0x25u
#define USB_MSD_SCSI_READ_10                            0x28u
#define USB_MSD_SCSI_REQUEST_SENSE                      0x03u
#define USB_MSD_SCSI_TEST_UNIT_READY                    0x00u
#define USB_MSD_SCSI_WRITE_10                           0x2Au
#define USB_MSD_SCSI_MODE_SENSE_6                       0x1Au
#define USB_MSD_SCSI_MODE_SELECT_6                      0x15u
#define USB_MSD_SCSI_PREVENT_ALLW_MDM_RMVL              0x1Eu
#define USB_MSD_SCSI_VERIFY_10                          0x2Fu
#define USB_MSD_SCSI_INVALID_COMMAND_CODE               0xFFu

/***************************************************************************//**
  Additional sense codes
 */
#define SC_NO_SENSE 				                    0x00u
#define SC_RECOVERED_ERR			                    0x01u
#define SC_NOT_READY 				                    0x02u
#define SC_MEDIUM_ERROR				                    0x03u
#define SC_HARDWARE_ERR				                    0x04u
#define SC_ILLEGAL_REQUEST 			                    0x05u
#define SC_UNIT_ATTENTION 			                    0x06u
#define SC_DATA_PROTECT	 			                    0x07u
#define SC_BLANK_CHECK	 			                    0x08u
#define SC_VENDOR_SPECIFIC 			                    0x09u
#define SC_COPY_ABORTED	 			                    0x0Au
#define SC_ABORTED_COMMAND 			                    0x0Bu
/*0x0C is obsolete*/
#define SC_VOLUME_OVERFLOW 			                    0x0Du
#define SC_MISCOMPARE	 			                    0x0Eu
/*0x0F is reserved*/

#define ASC_INVALID_CDB                                 0x20
#define ASC_INVALID_FIELED_IN_COMMAND                   0x24
#define ASC_PARAMETER_LIST_LENGTH_ERROR                 0x1A
#define ASC_INVALID_FIELD_IN_PARAMETER_LIST             0x26
#define ASC_ADDRESS_OUT_OF_RANGE                        0x21
#define ASC_MEDIUM_NOT_PRESENT                          0x3A
#define ASC_MEDIUM_HAS_CHANGED	                        0x28
#define ASC_WRITE_PROTECTED                             0x27 
#define ASC_UNRECOVERED_READ_ERROR			            0x11
#define ASC_WRITE_FAULT							        0x03 

/*******************************************************************************
 * Public constants andd data strucures from CDC class driver.
 */
 
 /***************************************************************************//**
 The usb_msd_state_t provides a type to identify the current state of the CDC class 
 driver.
	USB_MSD_NOT_CONFIGURED -- USB MSD class driver is not configured and it cannot
							  perform data transfers.
	USB_MSD_CONFIGURED     -- USB MSD class driver is configured by Host and it 
							  can perform data transfers.

	Note:    Application must not access the Flash media (on board SPI Flash in 
	this case) while the MSD class driver is in USB_MSD_CONFIGURED state. 
	This is because; the USB host might be performing read/write operation on the 
	Flash media. If application also attempts to access Flash media, the data would 
	be corrupted and host will get misleading information from Flash Media.
 */
typedef enum usb_msd_state {
	USB_MSD_NOT_CONFIGURED,
    USB_MSD_CONFIGURED,
}usb_msd_state_t;

/***************************************************************************//**
 */
typedef enum mss_usb_msd_bot_events {
	BOT_EVENT_IDLE,
	BOT_EVENT_DATAIN,
	BOT_EVENT_DATAOUT,
	BOT_EVENT_DATAIN_ERROR,
	BOT_EVENT_DATAOUT_ERROR
} mss_usb_msd_bot_events_t;

typedef enum mss_usb_msd_csw_status {
	SCSI_COMMAND_PASSED,
	SCSI_COMMAND_FAILED,	
	SCSI_COMMAND_PHASE_ERR
} mss_usb_msd_csw_status_t;

typedef enum mss_usb_msd_bot_state {
	BOT_IDLE,
	BOT_DATA_OUT,
	BOT_DATA_IN,
	BOT_SEND_STATUS
} mss_usb_msd_bot_state_t;

/***************************************************************************//**
  Command Block Warpper (CBW).
 */
typedef struct mss_usb_msd_cbw {
    uint32_t signature;
    uint32_t tag;
    uint32_t xfr_length;
    uint8_t flags;
    uint8_t lun;
    uint8_t cmd_length;
    uint8_t cmd_block[16];
} mss_usb_msd_cbw_t;

/***************************************************************************//**
  Command Status Wrapper (CSW).
 */
typedef struct mss_usb_msd_csw {
    uint32_t signature;
    uint32_t tag;
    uint32_t data_residue;
    mss_usb_msd_csw_status_t status;
} mss_usb_msd_csw_t;

/***************************************************************************//**
  SCSI inquiry response.
 */
typedef struct scsi_inquiry_response {
    uint8_t peripheral;
    uint8_t removable;
    uint8_t version;
    uint8_t resp_data_format;
    uint8_t additional_length;
    uint8_t sccstp;
    uint8_t bqueetc;
    uint8_t cmd_que;
    uint8_t vendor_id[8];
    uint8_t product_id[16];
    uint8_t product_rev[4];
} scsi_inquiry_response_t;

typedef struct scsi_request_sense_response {
    const uint8_t response_code;
    const uint8_t obsolete;
	uint8_t sense_key;
    const uint8_t info[4];
    const uint8_t additional_length;
    const uint8_t command_info[4];
	uint8_t asc;
    const uint8_t unused[5];
} scsi_request_sense_response_t;

typedef struct mss_usb_msd_media {
    uint8_t (* msd_media_init)( uint8_t lun );
    uint8_t (* msd_media_get_capacity)( uint8_t lun, uint32_t *last_block_lba, uint32_t *block_size );
    uint8_t (* msd_media_is_ready)( uint8_t lun );
    uint8_t (* msd_media_is_write_protected)( uint8_t lun );
    uint32_t (* msd_media_read)( uint8_t lun, uint8_t **buf, uint32_t blk_addr, uint32_t len );
    uint8_t* (* msd_media_acquire_write_buf)( uint8_t lun, uint32_t blk_addr, uint32_t *len );
    uint32_t (* msd_media_write_ready)( uint8_t lun, uint32_t blk_addr, uint32_t len );
    uint8_t (* msd_media_get_max_lun)( void );
    uint8_t* (* msd_media_inquiry)( uint8_t lun, uint32_t *len );
} mss_usb_msd_media_t;

typedef struct mss_usb_msd_lun_capacity {
	uint8_t last_lba_msb;
	uint8_t last_lba_2;
	uint8_t last_lba_1;
	uint8_t last_lba_lsb;
	uint8_t block_size_msb;
	uint8_t block_size_2;
	uint8_t block_size_1;
	uint8_t block_size_lsb;
	uint32_t blk_sz_len;
} mss_usb_msd_lun_capacity_t;

#ifdef __cplusplus
}
#endif

/***************************************************************************//**
 * Exported functions from MSD class driver.
 */

/***************************************************************************//**
  @brief MSS_USB_MSD_init()
	This API must be used by the application to start initialize the MSD class 
	driver. A pointer to the structure of type mss_usb_msd_media_t must be passed
	as a parameter to this function.  
	
  @param media_ops
	A pointer to the structure of type mss_usb_msd_media_t. This will be used by 
	the MSD class driver to call the call-back functions implemented by application.  

  @return
	This function does not return a value.
  
  Example:
  @code
  @endcode
*/
void
MSS_USB_MSD_init
(
    mss_usb_msd_media_t* media_ops
);

/***************************************************************************//**
  @brief MSS_USB_MSD_init()
	This API can be used by the application to know the current state of the MSD
	class driver.  
	
  @param media_ops
	This function does not take a parameter.  

  @return
	This function returns a value of type usb_msd_state_t indicating the current 
	state of the MSD class driver.
  
  Example:
  @code
  @endcode
*/
usb_msd_state_t
MSS_USB_MSD_get_state
(
    void
);

#endif  /* __MSS_USB_DEVICE_MSD_H_ */

