/*******************************************************************************
 * (c) Copyright 2011 Microsemi Corporation.  All rights reserved.
 *
 * National's DP83848 PHY interface driver source code.
 *
 * SVN $Revision: 3818 $
 * SVN $Date: 2011-09-09 17:31:01 +0530 (Fri, 09 Sep 2011) $
 */
#include <stdio.h>
#include "phy.h"
#include "dp83848_phy.h"
#include "mss_ethernet_mac_types.h"
#include "mss_ethernet_mac.h"

#ifdef __cplusplus
extern "C" {
#endif
/**************************************************************************/
/* Preprocessor Macros                                                    */
/**************************************************************************/

/**************************************************************************/
/* Private Function definitions                                           */
/**************************************************************************/

/**************************************************************************/
/* Public Function definitions                                            */
/**************************************************************************/

/**************************************************************************//**
 * See dp83848_phy.h for details of how to use this function.
 */
return_t dp83848_init(MAC_cfg_t * cfg)
{
	return_t status = FAILED;
    uint32_t temp;

    /* Reset the PHY interface */
    temp = MSS_MAC_read_phy_reg(DP83848_PHY_ADDR, MII_BMCR);
	temp |= BMCR_RESET;
	status = MSS_MAC_write_phy_reg(DP83848_PHY_ADDR, MII_BMCR, temp);

    /* 
     * Disable loopback. 
     * Enable auto negotiation and restart auto negotiation.
     */
    temp &= ~BMCR_LOOPBACK;
    if( cfg->auto_negotiation == MSS_MAC_AUTO_NEGOTIATION_ENABLE)
        temp |= (BMCR_ANENABLE | BMCR_ANRESTART);
    status = status && MSS_MAC_write_phy_reg(DP83848_PHY_ADDR, MII_BMCR, temp);
    
    return status;
}

/**************************************************************************//**
 * See dp83848_phy.h for details of how to use this function.
 */
return_t dp83848_config_aneg(MAC_cfg_t * cfg)
{
	return_t status = 0u;

    if(0u < phy_config_aneg(DP83848, cfg))
        status = SUCCESS;
    else
        status = FAILED;
    return status;
}

/**************************************************************************//**
 * See dp83848_phy.h for details of how to use this function.
 */
return_t dp83848_read_status(MAC_cfg_t * cfg)
{
	return_t status = FAILED;
    uint16_t physts = 0u;

    /* Update the link, force set the PHY if there
	 * was an error */
	status = phy_update_link( DP83848 );
	if (SUCCESS == status)
    {
		physts = MSS_MAC_read_phy_reg(DP83848_PHY_ADDR,
                                      DP83848_PHY_STATUS);
		if (0u == physts)
			return FAILED;

		if (physts & DP83848_PHY_STATUS_FULLDUPLEX)
			cfg->fullduplex = MSS_MAC_FULLDUPLEX_ENABLE;
		else
			cfg->fullduplex = MSS_MAC_FULLDUPLEX_DISABLE;

		if( physts & DP83848_PHY_STATUS_10 ) 
   			cfg->speed = MAC10MBPS;
        else
			cfg->speed = MAC100MBPS;
        status = SUCCESS;
	} 

    return status;
}

#ifdef __cplusplus
}
#endif

/******************************** END OF FILE ******************************/






