/*******************************************************************************
 * (c) Copyright 2012 Microsemi Corporation.  All rights reserved.
 *
 *  Microsemi Smartfusion2 USB device logical driver layer Implementation.	
 *  
 *
 *  Header file for mss_usb_device_hid.c
 *
 * SVN $Revision: 5027 $
 * SVN $Date: 2013-02-01 19:06:39 +0530 (Fri, 01 Feb 2013) $
 */

#ifndef __MSS_USB_DEVICE_HID_H_
#define __MSS_USB_DEVICE_HID_H_

#include "../../CMSIS/m2sxxx.h"


/*******************************************************************************
 * Public constants andd ata strucures from HID class driver.
 */

/*******************************************************************************
 * User Descriptor lengths
 */
#define CONFIGURATION_DESCR_LENGTH                      9u
#define INTERFACE_DESCR_LENGTH                          9u
#define HID_DESCR_LENGTH                                9u
#define ENDPOINT_DESCR_LENGTH                           7u

/*******************************************************************************
 * Full configuration descriptor length. APPLICATION/CONFIGURATION SPECIFIC!!!
 */
#define FULL_CONFIG_DESCR_LENGTH    (CONFIGURATION_DESCR_LENGTH + \
                                     INTERFACE_DESCR_LENGTH + \
                                     HID_DESCR_LENGTH + ENDPOINT_DESCR_LENGTH)

/***************************************************************************//**
  The usb_cdc_state_t is used to provide information of current state of the
  HID class driver.
  USB_HID_NOT_CONFIGURED -- USB HID class driver is not configured and it can 
                            not perform	data transfers.
  USB_HID_CONFIGURED    --  USB HID class driver is configured by Host and it 
                            can perform data data transfers.
  USB_HID_TRANSMITTING  --  USB HID class driver is configured by Host and is 
                            transmitting data.
  USB_HID_RECEIVING  --     USB HID class driver is configured by Host and is 
                            receiving data.
*/
typedef enum usb_hid_state {
	USB_HID_NOT_CONFIGURED,
    USB_HID_CONFIGURED,
	USB_HID_TRANSMITTING,
    USB_HID_RECEIVING
}usb_hid_state_t;

/***************************************************************************//**
The hid_report_t provides the prototype for the report to be sent to the host. 
Application must provide report data using this type.

buttons
This parameter provides current state of the Button0 to Button5 on mouse in 
bit0 to bit5 of this parameter.

x_move
This parameter provides current x co-ordinate of the mouse

y_move
This parameter provides current y co-ordinate of the mouse

wheel
This parameter provides current wheel position of the mouse
*/
typedef struct hid_report {
    int8_t  buttons;
    int8_t  x_move;
    int8_t  y_move;
    int8_t  wheel;
} hid_report_t;

/*Prototype for the report handler function*/
typedef uint8_t* (*get_hid_report_handler_t)( uint8_t* );

/***************************************************************************//**
* Exported function from HID Class driver.
*******************************************************************************/
/***************************************************************************//**
  @brief MSS_USB_HID_tx_report()
   This API can be used by application to provide the buffer containing the 
   report data to the HID class driver. The report data will be loaded into 
   the TX endpoint and will be transmitted when the next IN packet arrives 
	from host.
	
  @param  buf
   Address of the buffer containing the HID report data.
	
  @param  length
   Number of bytes to be transmitted
	  
  @return
   This function return a non-zero value if the function was successfully executed	
  
  Example:
  @code
  @endcode
*/
uint32_t
MSS_USB_HID_tx_report
(
    uint8_t* buf,
    uint32_t length
);

/***************************************************************************//**
  @brief MSS_USB_HID_tx_done()
	This API can be used by application to know if the transmission of the 
	previously provided report is complete.
  @param 
	This function takes no parameters.
	  
  @return
	This function returns a non-zero value to indicate the previously provided
	report is successfully stransmitted. Zero value indicates that the 
	transmission is not complete. 
  
  Example:
  @code
  @endcode
*/
uint32_t
MSS_USB_HID_tx_done
(
    void
);

/***************************************************************************//**
  @brief MSS_USB_HID_init()
   This API must be used by application to initialize the HID class driver before
   transmitting the report data.	
  @param 
   This function does not take any parameters.
  @return
   This function does not return any value.
  Example:
  @code
  @endcode
*/
void
MSS_USB_HID_init
(
    void
);
#endif  /* __MSS_USB_DEVICE_HID_H_ */
